@@ -9,14 +9,15 @@ mod aarch64_apple_darwin;
 mod aarch64_unknown_linux_gnu;
 mod aarch64_unknown_linux_musl;
 mod i686_unknown_linux_gnu;
+mod riscv64gc_unknown_linux_gnu;
 mod x86_64_apple_darwin;
 mod x86_64_unknown_linux_gnu;
 mod x86_64_unknown_linux_musl;
 
 use crate::{
     cargo_env, effective_target, emit_warning, env_var_to_bool, execute_command, get_crate_cflags,
-    is_no_asm, option_env, out_dir, requested_c_std, target, target_arch, target_env, target_os,
-    target_vendor, CStdRequested, OutputLibType,
+    is_no_asm, optional_env_optional_crate_target, out_dir, requested_c_std, set_env_for_target,
+    target, target_arch, target_env, target_os, target_vendor, CStdRequested, OutputLibType,
 };
 use std::path::PathBuf;
 
@@ -27,7 +28,7 @@ pub(crate) struct CcBuilder {
     output_lib_type: OutputLibType,
 }
 
-use std::{env, fs};
+use std::fs;
 
 pub(crate) struct Library {
     name: &'static str,
@@ -40,6 +41,7 @@ enum PlatformConfig {
     aarch64_apple_darwin,
     aarch64_unknown_linux_gnu,
     aarch64_unknown_linux_musl,
+    riscv64gc_unknown_linux_gnu,
     x86_64_apple_darwin,
     x86_64_unknown_linux_gnu,
     x86_64_unknown_linux_musl,
@@ -54,6 +56,9 @@ impl PlatformConfig {
             PlatformConfig::aarch64_unknown_linux_musl => {
                 aarch64_unknown_linux_musl::CRYPTO_LIBRARY
             }
+            PlatformConfig::riscv64gc_unknown_linux_gnu => {
+                riscv64gc_unknown_linux_gnu::CRYPTO_LIBRARY
+            }
             PlatformConfig::x86_64_apple_darwin => x86_64_apple_darwin::CRYPTO_LIBRARY,
             PlatformConfig::x86_64_unknown_linux_gnu => x86_64_unknown_linux_gnu::CRYPTO_LIBRARY,
             PlatformConfig::x86_64_unknown_linux_musl => x86_64_unknown_linux_musl::CRYPTO_LIBRARY,
@@ -67,6 +72,7 @@ impl PlatformConfig {
             "aarch64-apple-darwin" => Some(PlatformConfig::aarch64_apple_darwin),
             "aarch64-unknown-linux-gnu" => Some(PlatformConfig::aarch64_unknown_linux_gnu),
             "aarch64-unknown-linux-musl" => Some(PlatformConfig::aarch64_unknown_linux_musl),
+            "riscv64gc-unknown-linux-gnu" => Some(PlatformConfig::riscv64gc_unknown_linux_gnu),
             "x86_64-apple-darwin" => Some(PlatformConfig::x86_64_apple_darwin),
             "x86_64-unknown-linux-gnu" => Some(PlatformConfig::x86_64_unknown_linux_gnu),
             "x86_64-unknown-linux-musl" => Some(PlatformConfig::x86_64_unknown_linux_musl),
@@ -82,6 +88,74 @@ impl Default for PlatformConfig {
     }
 }
 
+#[allow(clippy::upper_case_acronyms)]
+pub(crate) enum BuildOption {
+    STD(String),
+    FLAG(String),
+    DEFINE(String, String),
+    INCLUDE(PathBuf),
+}
+impl BuildOption {
+    fn std<T: ToString + ?Sized>(val: &T) -> Self {
+        Self::STD(val.to_string())
+    }
+    fn flag<T: ToString + ?Sized>(val: &T) -> Self {
+        Self::FLAG(val.to_string())
+    }
+    fn flag_if_supported<T: ToString + ?Sized>(cc_build: &cc::Build, flag: &T) -> Option<Self> {
+        if let Ok(true) = cc_build.is_flag_supported(flag.to_string()) {
+            Some(Self::FLAG(flag.to_string()))
+        } else {
+            None
+        }
+    }
+
+    fn define<K: ToString + ?Sized, V: ToString + ?Sized>(key: &K, val: &V) -> Self {
+        Self::DEFINE(key.to_string(), val.to_string())
+    }
+
+    fn include<P: Into<PathBuf>>(path: P) -> Self {
+        Self::INCLUDE(path.into())
+    }
+
+    fn apply_cc<'a>(&self, cc_build: &'a mut cc::Build) -> &'a mut cc::Build {
+        match self {
+            BuildOption::STD(val) => cc_build.std(val),
+            BuildOption::FLAG(val) => cc_build.flag(val),
+            BuildOption::DEFINE(key, val) => cc_build.define(key, Some(val.as_str())),
+            BuildOption::INCLUDE(path) => cc_build.include(path.as_path()),
+        }
+    }
+
+    pub(crate) fn apply_cmake<'a>(
+        &self,
+        cmake_cfg: &'a mut cmake::Config,
+        is_like_msvc: bool,
+    ) -> &'a mut cmake::Config {
+        if is_like_msvc {
+            match self {
+                BuildOption::STD(val) => cmake_cfg.define(
+                    "CMAKE_C_STANDARD",
+                    val.to_ascii_lowercase().strip_prefix('c').unwrap_or("11"),
+                ),
+                BuildOption::FLAG(val) => cmake_cfg.cflag(val),
+                BuildOption::DEFINE(key, val) => cmake_cfg.cflag(format!("/D{key}={val}")),
+                BuildOption::INCLUDE(path) => cmake_cfg.cflag(format!("/I{}", path.display())),
+            }
+        } else {
+            match self {
+                BuildOption::STD(val) => cmake_cfg.define(
+                    "CMAKE_C_STANDARD",
+                    val.to_ascii_lowercase().strip_prefix('c').unwrap_or("11"),
+                ),
+                BuildOption::FLAG(val) => cmake_cfg.cflag(val),
+                BuildOption::DEFINE(key, val) => cmake_cfg.cflag(format!("-D{key}={val}")),
+                BuildOption::INCLUDE(path) => cmake_cfg.cflag(format!("-I{}", path.display())),
+            }
+        }
+    }
+}
+
 impl CcBuilder {
     pub(crate) fn new(
         manifest_dir: PathBuf,
@@ -97,57 +171,44 @@ impl CcBuilder {
         }
     }
 
-    pub(crate) fn create_builder(&self) -> cc::Build {
-        let mut cc_build = cc::Build::default();
-        cc_build.out_dir(&self.out_dir).cpp(false);
+    pub(crate) fn collect_universal_build_options(
+        &self,
+        cc_build: &cc::Build,
+    ) -> (bool, Vec<BuildOption>) {
+        let mut build_options: Vec<BuildOption> = Vec::new();
 
-        let compiler = cc_build.get_compiler();
-        if compiler.is_like_gnu() || compiler.is_like_clang() {
-            cc_build.flag("-Wno-unused-parameter");
-            if target_os() == "linux"
-                || target_os().ends_with("bsd")
-                || target_env() == "gnu"
-                || target_env() == "musl"
-            {
-                cc_build.define("_XOPEN_SOURCE", "700").flag("-pthread");
-            }
-        }
+        let compiler_is_msvc = {
+            let compiler = cc_build.get_compiler();
+            !compiler.is_like_gnu() && !compiler.is_like_clang()
+        };
 
-        self.add_includes(&mut cc_build);
-
-        cc_build
-    }
-
-    pub(crate) fn prepare_builder(&self) -> cc::Build {
-        let mut cc_build = self.create_builder();
         match requested_c_std() {
             CStdRequested::C99 => {
-                cc_build.std("c99");
+                build_options.push(BuildOption::std("c99"));
             }
             CStdRequested::C11 => {
-                cc_build.std("c11");
+                build_options.push(BuildOption::std("c11"));
             }
             CStdRequested::None => {
-                if target_env() == "msvc" && target_arch() == "aarch64" {
-                    // clang-cl (not "clang") will be used.
-                } else if self.compiler_check("c11", "") {
-                    cc_build.std("c11");
+                if self.compiler_check("c11") {
+                    build_options.push(BuildOption::std("c11"));
                 } else {
-                    cc_build.std("c99");
+                    build_options.push(BuildOption::std("c99"));
                 }
             }
         }
 
-        if let Some(cc) = option_env("CC") {
-            emit_warning(&format!("CC environment variable set: {}", cc.clone()));
+        if let Some(cc) = optional_env_optional_crate_target("CC") {
+            set_env_for_target("CC", &cc);
         }
-        if let Some(cxx) = option_env("CXX") {
-            emit_warning(&format!("CXX environment variable set: {}", cxx.clone()));
+        if let Some(cxx) = optional_env_optional_crate_target("CXX") {
+            set_env_for_target("CXX", &cxx);
         }
 
-        let compiler = cc_build.get_compiler();
-        if target_arch() == "x86" && (compiler.is_like_clang() || compiler.is_like_gnu()) {
-            cc_build.flag_if_supported("-msse2");
+        if target_arch() == "x86" && !compiler_is_msvc {
+            if let Some(option) = BuildOption::flag_if_supported(cc_build, "-msse2") {
+                build_options.push(option);
+            }
         }
 
         let opt_level = cargo_env("OPT_LEVEL");
@@ -155,7 +216,7 @@ impl CcBuilder {
             "0" | "1" | "2" => {
                 if is_no_asm() {
                     emit_warning("AWS_LC_SYS_NO_ASM found. Disabling assembly code usage.");
-                    cc_build.define("OPENSSL_NO_ASM", "1");
+                    build_options.push(BuildOption::define("OPENSSL_NO_ASM", "1"));
                 }
             }
             _ => {
@@ -163,97 +224,167 @@ impl CcBuilder {
                     !is_no_asm(),
                     "AWS_LC_SYS_NO_ASM only allowed for debug builds!"
                 );
-                if compiler.is_like_gnu() || compiler.is_like_clang() {
+                if !compiler_is_msvc {
                     let flag = format!("-ffile-prefix-map={}=", self.manifest_dir.display());
                     if let Ok(true) = cc_build.is_flag_supported(&flag) {
                         emit_warning(&format!("Using flag: {}", &flag));
-                        cc_build.flag(flag);
+                        build_options.push(BuildOption::flag(&flag));
                     } else {
                         emit_warning("NOTICE: Build environment source paths might be visible in release binary.");
                         let flag = format!("-fdebug-prefix-map={}=", self.manifest_dir.display());
                         if let Ok(true) = cc_build.is_flag_supported(&flag) {
                             emit_warning(&format!("Using flag: {}", &flag));
-                            cc_build.flag(flag);
+                            build_options.push(BuildOption::flag(&flag));
                         }
                     }
                 }
             }
         }
 
-        if !get_crate_cflags().is_empty() {
-            let cflags = get_crate_cflags();
-            emit_warning(&format!(
-                "AWS_LC_SYS_CFLAGS found. Setting CFLAGS: '{cflags}'"
-            ));
-            env::set_var("CFLAGS", cflags);
-        }
-
         if target_os() == "macos" {
             // This compiler error has only been seen on MacOS x86_64:
             // ```
             // clang: error: overriding '-mmacosx-version-min=13.7' option with '--target=x86_64-apple-macosx14.2' [-Werror,-Woverriding-t-option]
             // ```
-            cc_build.flag_if_supported("-Wno-overriding-t-option");
-            cc_build.flag_if_supported("-Wno-overriding-option");
+            if let Some(option) =
+                BuildOption::flag_if_supported(cc_build, "-Wno-overriding-t-option")
+            {
+                build_options.push(option);
+            }
+            if let Some(option) = BuildOption::flag_if_supported(cc_build, "-Wno-overriding-option")
+            {
+                build_options.push(option);
+            }
         }
-
-        cc_build
+        (compiler_is_msvc, build_options)
     }
 
-    fn add_includes(&self, cc_build: &mut cc::Build) {
+    pub fn collect_cc_only_build_options(&self, cc_build: &cc::Build) -> Vec<BuildOption> {
+        let mut build_options: Vec<BuildOption> = Vec::new();
+        let is_like_msvc = {
+            let compiler = cc_build.get_compiler();
+            !compiler.is_like_gnu() && !compiler.is_like_clang()
+        };
+        if !is_like_msvc {
+            build_options.push(BuildOption::flag("-Wno-unused-parameter"));
+            if target_os() == "linux"
+                || target_os().ends_with("bsd")
+                || target_env() == "gnu"
+                || target_env() == "musl"
+            {
+                build_options.push(BuildOption::define("_XOPEN_SOURCE", "700"));
+                build_options.push(BuildOption::flag("-pthread"));
+            }
+        }
+
+        self.add_includes(&mut build_options);
+
+        build_options
+    }
+
+    fn add_includes(&self, build_options: &mut Vec<BuildOption>) {
         // The order of includes matters
         if let Some(prefix) = &self.build_prefix {
-            cc_build
-                .define("BORINGSSL_IMPLEMENTATION", "1")
-                .define("BORINGSSL_PREFIX", prefix.as_str());
-            cc_build.include(self.manifest_dir.join("generated-include"));
+            build_options.push(BuildOption::define("BORINGSSL_IMPLEMENTATION", "1"));
+            build_options.push(BuildOption::define("BORINGSSL_PREFIX", prefix.as_str()));
+            build_options.push(BuildOption::include(
+                self.manifest_dir.join("generated-include"),
+            ));
         }
-        cc_build
-            .include(self.manifest_dir.join("include"))
-            .include(self.manifest_dir.join("aws-lc").join("include"))
-            .include(
-                self.manifest_dir
-                    .join("aws-lc")
-                    .join("third_party")
-                    .join("s2n-bignum")
-                    .join("include"),
-            );
+        build_options.push(BuildOption::include(self.manifest_dir.join("include")));
+        build_options.push(BuildOption::include(
+            self.manifest_dir.join("aws-lc").join("include"),
+        ));
+        build_options.push(BuildOption::include(
+            self.manifest_dir
+                .join("aws-lc")
+                .join("third_party")
+                .join("s2n-bignum")
+                .join("include"),
+        ));
+        build_options.push(BuildOption::include(
+            self.manifest_dir
+                .join("aws-lc")
+                .join("third_party")
+                .join("jitterentropy")
+                .join("jitterentropy-library"),
+        ));
     }
 
-    fn add_all_files(&self, lib: &Library, cc_build: &mut cc::Build) {
+    pub fn create_builder(&self) -> cc::Build {
+        let mut cc_build = cc::Build::new();
+        let build_options = self.collect_cc_only_build_options(&cc_build);
+        for option in build_options {
+            option.apply_cc(&mut cc_build);
+        }
+        cc_build
+    }
+
+    pub fn prepare_builder(&self) -> cc::Build {
+        let mut cc_build = self.create_builder();
+        let (_, build_options) = self.collect_universal_build_options(&cc_build);
+        for option in build_options {
+            option.apply_cc(&mut cc_build);
+        }
+        let cflags = get_crate_cflags();
+        if !cflags.is_empty() {
+            set_env_for_target("CFLAGS", cflags);
+        }
+        cc_build
+    }
+
+    fn add_all_files(&self, lib: &Library, cc_build: &mut cc::Build) -> Vec<PathBuf> {
         use core::str::FromStr;
         cc_build.file(PathBuf::from_str("rust_wrapper.c").unwrap());
 
+        // s2n_bignum is compiled separately due to needing extra flags
+        let mut s2n_bignum_builder = cc_build.clone();
+        s2n_bignum_builder.flag(format!(
+            "--include={}",
+            self.manifest_dir
+                .join("generated-include")
+                .join("openssl")
+                .join("boringssl_prefix_symbols_asm.h")
+                .display()
+        ));
+        s2n_bignum_builder.define("S2N_BN_HIDE_SYMBOLS", "1");
+        let cc_preprocessor = self.create_builder();
         for source in lib.sources {
             let source_path = self.manifest_dir.join("aws-lc").join(source);
-            let is_asm = std::path::Path::new(source)
-                .extension()
-                .map_or(false, |ext| ext.eq("S"));
-            if is_asm && target_vendor() == "apple" && target_arch() == "aarch64" {
-                let mut cc_preprocessor = self.create_builder();
-                cc_preprocessor.file(source_path);
-                let preprocessed_asm = String::from_utf8(cc_preprocessor.expand()).unwrap();
-                let preprocessed_asm = preprocessed_asm.replace(';', "\n\t");
-                let asm_output_path = self.out_dir.join(source);
-                fs::create_dir_all(asm_output_path.parent().unwrap()).unwrap();
-                fs::write(asm_output_path.clone(), preprocessed_asm).unwrap();
-                cc_build.file(asm_output_path);
+            let is_s2n_bignum = std::path::Path::new(source).starts_with("third_party/s2n-bignum");
+
+            if is_s2n_bignum {
+                let asm_output_path = if target_vendor() == "apple" && target_arch() == "aarch64" {
+                    let asm_output_path = self.out_dir.join(source);
+                    let mut cc_preprocessor = cc_preprocessor.clone();
+                    cc_preprocessor.file(source_path);
+                    let preprocessed_asm = String::from_utf8(cc_preprocessor.expand()).unwrap();
+                    let preprocessed_asm = preprocessed_asm.replace(';', "\n\t");
+                    fs::create_dir_all(asm_output_path.parent().unwrap()).unwrap();
+                    fs::write(asm_output_path.clone(), preprocessed_asm).unwrap();
+                    asm_output_path
+                } else {
+                    source_path.clone()
+                };
+                s2n_bignum_builder.file(asm_output_path);
             } else {
                 cc_build.file(source_path);
             }
         }
+        s2n_bignum_builder.compile_intermediates()
     }
 
     fn build_library(&self, lib: &Library) {
         let mut cc_build = self.prepare_builder();
-
-        self.add_all_files(lib, &mut cc_build);
-
         for flag in lib.flags {
             cc_build.flag(flag);
         }
-        self.run_compiler_checks();
+        self.run_compiler_checks(&mut cc_build);
 
+        let object_files = self.add_all_files(lib, &mut cc_build);
+        for object in object_files {
+            cc_build.object(object);
+        }
         if let Some(prefix) = &self.build_prefix {
             cc_build.compile(format!("{}_crypto", prefix.as_str()).as_str());
         } else {
@@ -264,10 +395,9 @@ impl CcBuilder {
     // This performs basic checks of compiler capabilities and sets an appropriate flag on success.
     // This should be kept in alignment with the checks performed by AWS-LC's CMake build.
     // See: https://github.com/search?q=repo%3Aaws%2Faws-lc%20check_compiler&type=code
-    fn compiler_check(&self, basename: &str, flag: &str) -> bool {
+    fn compiler_check(&self, basename: &str) -> bool {
         let mut ret_val = false;
         let output_dir = self.out_dir.join(format!("out-{basename}"));
-        let mut cc_build = self.create_builder();
         let source_file = self
             .manifest_dir
             .join("aws-lc")
@@ -278,13 +408,14 @@ impl CcBuilder {
             emit_warning("######");
             emit_warning("###### WARNING: MISSING GIT SUBMODULE ######");
             emit_warning(&format!(
-                "  -- Did you initialize the repo's git submodules? Unable to find source file: {:?}.",
-                &source_file
+                "  -- Did you initialize the repo's git submodules? Unable to find source file: {}.",
+                source_file.display()
             ));
             emit_warning("  -- run 'git submodule update --init --recursive' to initialize.");
             emit_warning("######");
             emit_warning("######");
         }
+        let mut cc_build = cc::Build::default();
         cc_build
             .file(source_file)
             .warnings_into_errors(true)
@@ -297,13 +428,10 @@ impl CcBuilder {
         let result = cc_build.try_compile_intermediates();
 
         if result.is_ok() {
-            if !flag.is_empty() {
-                cc_build.define(flag, "1");
-            }
             ret_val = true;
         }
         if fs::remove_dir_all(&output_dir).is_err() {
-            emit_warning(&format!("Failed to remove {:?}", &output_dir));
+            emit_warning(&format!("Failed to remove {}", output_dir.display()));
         }
         emit_warning(&format!(
             "Compilation of '{basename}.c' {} - {:?}.",
@@ -347,14 +475,14 @@ impl CcBuilder {
             execute_command(memcmp_compiler.path().as_os_str(), memcmp_args.as_slice());
         assert!(
             memcmp_compile_result.status,
-            "COMPILER: {:?}\
+            "COMPILER: {}\
             ARGS: {:?}\
             EXECUTED: {}\
             ERROR: {}\
             OUTPUT: {}\
             Failed to compile {basename}
             ",
-            memcmp_compiler.path(),
+            memcmp_compiler.path().display(),
             memcmp_args.as_slice(),
             memcmp_compile_result.executed,
             memcmp_compile_result.stderr,
@@ -381,9 +509,13 @@ impl CcBuilder {
         }
         let _ = fs::remove_file(exec_path);
     }
-    fn run_compiler_checks(&self) {
-        self.compiler_check("stdalign_check", "AWS_LC_STDALIGN_AVAILABLE");
-        self.compiler_check("builtin_swap_check", "AWS_LC_BUILTIN_SWAP_SUPPORTED");
+    fn run_compiler_checks(&self, cc_build: &mut cc::Build) {
+        if self.compiler_check("stdalign_check") {
+            cc_build.define("AWS_LC_STDALIGN_AVAILABLE", Some("1"));
+        }
+        if self.compiler_check("builtin_swap_check") {
+            cc_build.define("AWS_LC_BUILTIN_SWAP_SUPPORTED", Some("1"));
+        }
         self.memcmp_check();
     }
 }
