@@ -39,8 +39,8 @@ use crate::error::{OTelSdkError, OTelSdkResult};
 /// use opentelemetry_sdk::trace::SdkTracerProvider;
 /// use opentelemetry::trace::Tracer;
 ///
-/// fn init_tracing() -> TracerProvider {
-///     let provider = TracerProvider::default();
+/// fn init_tracing() -> SdkTracerProvider {
+///     let provider = SdkTracerProvider::default();
 ///
 ///     // Set the provider to be used globally
 ///     let _ = global::set_tracer_provider(provider.clone());
@@ -198,8 +198,8 @@ impl SdkTracerProvider {
     /// use opentelemetry::global;
     /// use opentelemetry_sdk::trace::SdkTracerProvider;
     ///
-    /// fn init_tracing() -> TracerProvider {
-    ///     let provider = TracerProvider::default();
+    /// fn init_tracing() -> SdkTracerProvider {
+    ///     let provider = SdkTracerProvider::default();
     ///
     ///     // Set provider to be used as global tracer provider
     ///     let _ = global::set_tracer_provider(provider.clone());
@@ -213,10 +213,8 @@ impl SdkTracerProvider {
     ///     // create spans..
     ///
     ///     // force all spans to flush
-    ///     for result in provider.force_flush() {
-    ///         if let Err(err) = result {
-    ///             // .. handle flush error
-    ///         }
+    ///     if let Err(err) = provider.force_flush() {
+    ///         // .. handle flush error
     ///     }
     ///
     ///     // create more spans..
@@ -293,6 +291,7 @@ impl opentelemetry::trace::TracerProvider for SdkTracerProvider {
 pub struct TracerProviderBuilder {
     processors: Vec<Box<dyn SpanProcessor>>,
     config: crate::trace::Config,
+    resource: Option<Resource>,
 }
 
 impl TracerProviderBuilder {
@@ -308,7 +307,7 @@ impl TracerProviderBuilder {
     ///
     /// Processors are invoked in the order they are added.
     pub fn with_simple_exporter<T: SpanExporter + 'static>(self, exporter: T) -> Self {
-        let simple = SimpleSpanProcessor::new(Box::new(exporter));
+        let simple = SimpleSpanProcessor::new(exporter);
         self.with_span_processor(simple)
     }
 
@@ -346,15 +345,6 @@ impl TracerProviderBuilder {
         TracerProviderBuilder { processors, ..self }
     }
 
-    /// The sdk [`crate::trace::Config`] that this provider will use.
-    #[deprecated(
-        since = "0.27.1",
-        note = "Config is becoming a private type. Use Builder::with_{config_name}(resource) instead. ex: Builder::with_resource(resource)"
-    )]
-    pub fn with_config(self, config: crate::trace::Config) -> Self {
-        TracerProviderBuilder { config, ..self }
-    }
-
     /// Specify the sampler to be used.
     pub fn with_sampler<T: crate::trace::ShouldSample + 'static>(mut self, sampler: T) -> Self {
         self.config.sampler = Box::new(sampler);
@@ -410,18 +400,28 @@ impl TracerProviderBuilder {
     ///
     /// By default, if this option is not used, the default [Resource] will be used.
     ///
+    /// *Note*: Calls to this method are additive, each call merges the provided
+    /// resource with the previous one.
+    ///
     /// [Tracer]: opentelemetry::trace::Tracer
     pub fn with_resource(self, resource: Resource) -> Self {
-        TracerProviderBuilder {
-            config: self.config.with_resource(resource),
-            ..self
-        }
+        let resource = match self.resource {
+            Some(existing) => Some(existing.merge(&resource)),
+            None => Some(resource),
+        };
+
+        TracerProviderBuilder { resource, ..self }
     }
 
     /// Create a new provider from this configuration.
     pub fn build(self) -> SdkTracerProvider {
         let mut config = self.config;
 
+        // Now, we can update the config with the resource.
+        if let Some(resource) = self.resource {
+            config.resource = Cow::Owned(resource);
+        };
+
         // Standard config will contain an owned [`Resource`] (either sdk default or use supplied)
         // we can optimize the common case with a static ref to avoid cloning the underlying
         // resource data for each span.
@@ -462,8 +462,8 @@ mod tests {
         SERVICE_NAME, TELEMETRY_SDK_LANGUAGE, TELEMETRY_SDK_NAME, TELEMETRY_SDK_VERSION,
     };
     use crate::trace::provider::TracerProviderInner;
-    use crate::trace::SpanData;
     use crate::trace::{Config, Span, SpanProcessor};
+    use crate::trace::{SdkTracerProvider, SpanData};
     use crate::Resource;
     use opentelemetry::trace::{Tracer, TracerProvider};
     use opentelemetry::{Context, Key, KeyValue, Value};
@@ -730,6 +730,39 @@ mod tests {
         assert!(test_tracer_1.provider().is_shutdown());
     }
 
+    #[test]
+    fn with_resource_multiple_calls_ensure_additive() {
+        let resource = SdkTracerProvider::builder()
+            .with_resource(Resource::new(vec![KeyValue::new("key1", "value1")]))
+            .with_resource(Resource::new(vec![KeyValue::new("key2", "value2")]))
+            .with_resource(
+                Resource::builder_empty()
+                    .with_schema_url(vec![], "http://example.com")
+                    .build(),
+            )
+            .with_resource(Resource::new(vec![KeyValue::new("key3", "value3")]))
+            .build()
+            .inner
+            .config
+            .resource
+            .clone()
+            .into_owned();
+
+        assert_eq!(
+            resource.get(&Key::from_static_str("key1")),
+            Some(Value::from("value1"))
+        );
+        assert_eq!(
+            resource.get(&Key::from_static_str("key2")),
+            Some(Value::from("value2"))
+        );
+        assert_eq!(
+            resource.get(&Key::from_static_str("key3")),
+            Some(Value::from("value3"))
+        );
+        assert_eq!(resource.schema_url(), Some("http://example.com"));
+    }
+
     #[derive(Debug)]
     struct CountingShutdownProcessor {
         shutdown_count: Arc<AtomicU32>,
